package com.biotechvana.netools.internal;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.progress.IProgressService;
import org.eclipse.jface.fieldassist.AutoCompleteField;
import org.eclipse.jface.fieldassist.ComboContentAdapter;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.Section;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.biotechvana.commons.SharedImages;
import com.biotechvana.netools.IVariablesView;
import com.biotechvana.netools.projects.Dataset;
import com.biotechvana.netools.projects.IProjectsManager;
import com.biotechvana.netools.projects.NetworkDesign;
import com.biotechvana.netools.projects.Project;
import com.biotechvana.netools.projects.Variable;

import jakarta.annotation.PostConstruct;
import jakarta.inject.Inject;
import jakarta.inject.Named;

public class BlackListView implements IVariablesView {
	public static final Logger logger = LoggerFactory.getLogger(BlackListView.class);
	FormToolkit toolkit;
	ScrolledForm detailsForm;

	Dataset selected_dataset;
	Project activeProject;
	NetworkDesign activeDesign;
	IProjectsManager projectsManager;

	@Inject
	IProgressService progressService;

	@Inject
	UISynchronize sync;


	List<Variable> variables = new ArrayList<Variable>();
	Composite mainComposite;
	TableViewer tableViewer;



	
	@Inject
	@Optional 
	public void setActiveDesign(@Named(IProjectsManager.ACTIVE_DESIGN) NetworkDesign networkDesign) {
		if (networkDesign == null) {
			activeDesign = null;
			activeProject = null;
			variables = new ArrayList<>();
		}
		else {
			this.activeDesign = networkDesign;
			this.activeProject = networkDesign.getProject();
			this.variables = activeProject.getVariables();
		}
		sync.asyncExec(
				() -> updateUI()
			);
	}
	


	/**
	 * Update GUI in must be called in the display thread
	 */
	private void updateUI() {
		if (detailsForm == null || detailsForm.isDisposed()) {
			return;
		}
		logger.debug("Updating UI for the View");
		if (activeDesign != null) {
			detailsForm.setText("Network Designer: " + activeDesign.getDesignName());
			tableViewer.setInput(activeDesign.getWhitelist());
		} else {
			detailsForm.setText("Network Designer - No Active Design");
			tableViewer.setInput(new ArrayList<>());
		}
		 setComboValues();
		


	}

	private void renderMainSection() {
		tableViewer = new TableViewer(mainComposite, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.BORDER);
		tableViewer.getTable().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		tableViewer.getTable().setHeaderVisible(true);
		tableViewer.getTable().setLinesVisible(true);
		tableViewer.setContentProvider(new ArrayContentProvider());
		// tableViewer to display Arcs
		
		
		
		
		
		if (activeDesign != null) {
			tableViewer.setInput(activeDesign.getWhitelist());
		} else {
			tableViewer.setInput(new ArrayList<>());
		}
		
	}

	



	@PostConstruct
	public void createPartControl(Composite parent, MApplication app, IProjectsManager projectsManager) { 
		app.getContext().set(IVariablesView.class, this);
		this.projectsManager = projectsManager;
		toolkit = new FormToolkit(parent.getDisplay());

		detailsForm = toolkit.createScrolledForm(parent);
		detailsForm.setText("Network Designer");
		detailsForm.getBody().setLayout(new GridLayout(1, false));

		// Create filter section
		Section descSection = toolkit.createSection(detailsForm.getBody(), Section.TITLE_BAR);
		descSection.setText("Add/Remove edges from Blacklist ");
		descSection.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));

		Composite descComposite = toolkit.createComposite(descSection);
		descComposite.setLayout(new FillLayout());

		createDescSection(descComposite);

		descSection.setClient(descComposite);

		// Create variables section
		Section mainSection = toolkit.createSection(detailsForm.getBody(), Section.TITLE_BAR);
		mainSection.setText("Blacklist");
		mainSection.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		mainComposite = toolkit.createComposite(mainSection);
		mainComposite.setLayout(new GridLayout(1, false));
		renderMainSection();
		mainSection.setClient(mainComposite);
	}
	Combo comboVarFrom;
	Combo comboVarTo ;
	Button btnAdd;
	private void createDescSection(Composite descComposite) {
		Composite addForm = toolkit.createComposite(descComposite);
		addForm.setLayout(new GridLayout(3, false));
		addForm.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		toolkit.createLabel(addForm, "From: ");
        comboVarFrom = new Combo(addForm, SWT.BORDER | SWT.DROP_DOWN | SWT.SINGLE   | SWT.FLAT);
        comboVarFrom.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false, 2, 1));
        
		
        
        // to field
        toolkit.createLabel(addForm, "To: ");
        comboVarTo = new Combo(addForm, SWT.BORDER | SWT.DROP_DOWN | SWT.SINGLE   | SWT.FLAT);
        comboVarTo.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false, 2, 1));
        
        // add add button
        toolkit.createLabel(addForm, "");
        toolkit.createLabel(addForm, "");
        btnAdd = toolkit.createButton(addForm, "Add", SWT.PUSH);
        btnAdd.setLayoutData(new GridData(SWT.RIGHT, SWT.TOP, true, false));
        btnAdd.setImage(SharedImages.ADD_ITEM);
        btnAdd.setEnabled(false);
        
        SelectionAdapter updateAdapter =   new SelectionAdapter() {
   		 
			@Override
			public void widgetSelected(SelectionEvent e) {
				String from = comboVarFrom.getText();
				String to = comboVarTo.getText();
				logger.debug("Adding edge from {} to {}", from, to);
				if (varMap.containsKey(from) && varMap.containsKey(to)) {
					btnAdd.setEnabled(true);
				} else {
					btnAdd.setEnabled(false);
				}
			}
		};
		comboVarFrom.addSelectionListener(updateAdapter);
		comboVarTo.addSelectionListener(updateAdapter);
		
		ModifyListener modifyAdapter = new ModifyListener() {
			
			@Override
			public void modifyText(ModifyEvent e) {
				String from = comboVarFrom.getText();
				String to = comboVarTo.getText();
				logger.debug("Adding edge from {} to {}", from, to);
				if (varMap.containsKey(from) && varMap.containsKey(to)) {
					btnAdd.setEnabled(true);
				} else {
					btnAdd.setEnabled(false);
				}
			}
		};
		
		
		comboVarTo.addModifyListener(modifyAdapter);
		comboVarFrom.addModifyListener(modifyAdapter);
		
        
        setComboValues();
       
	}
	HashMap<String, Variable> varMap = new HashMap<>();

	protected void setComboValues() {
		
		// fill from combo and to combo
		comboVarFrom.removeAll();
		comboVarTo.removeAll();
		varMap.clear();
		
		comboVarTo.add("*");
		varMap.put("*", null);
		for (Variable v : variables) {
			comboVarFrom.add(v.getName());
			comboVarTo.add(v.getName());
			varMap.put(v.getName(), v);
		}
		if (activeDesign != null) {
            // btnAdd.setEnabled(true);
            comboVarFrom.setEnabled(true);
            comboVarTo.setEnabled(true);
            
            new AutoCompleteField(comboVarFrom, new ComboContentAdapter(),  comboVarFrom.getItems());
            new AutoCompleteField(comboVarTo, new ComboContentAdapter(), comboVarTo.getItems());

        }
		else {
        	btnAdd.setEnabled(false);
        	comboVarFrom.setEnabled(false);
        	comboVarTo.setEnabled(false);
        }
		
	}


}
