package com.biotechvana.netools.projects;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonBackReference;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonIdentityInfo;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.ObjectIdGenerators;

@JsonIdentityInfo(generator = ObjectIdGenerators.PropertyGenerator.class, property = "designID" , scope = NetworkDesign.class)
@JsonAutoDetect(fieldVisibility = JsonAutoDetect.Visibility.ANY, getterVisibility = JsonAutoDetect.Visibility.NONE)
public class NetworkDesign {

	private Set<Arc> whitelist = new HashSet<>();
	private Set<Arc> blacklist = new HashSet<>();


	private HashMap<String,List<String>> dataTypesWhitelist = new HashMap<>();
	private HashMap<String,List<String>> dataTypesBlacklist = new HashMap<>();

	
	String designID;
	
	String name;
	String designDescription;
	
	@JsonBackReference  //("project-design")
	Project project;
	
	
	@JsonCreator
	public NetworkDesign(@JsonProperty("name") String designName,@JsonProperty("designDescription")  String designDescription) {
		this.name = designName;
		this.designID = designName;
		this.designDescription = designDescription;
	}
	/**
	 * add to whitelist of the project
	 */
	public void addWhitelist(Variable source, Variable target) {
		Arc arc = new Arc(source, target);
		whitelist.add(arc);
	}
	public void addWhitelist(Variable source, Variable target, String reason) {
		Arc arc = new Arc(source, target);
		arc.setReason(reason);
		
		whitelist.add(arc);
	}

	/**
	 * remove from whitelist of the project
	 */
	public void removeWhitelist(Variable source, Variable target) {
		Arc arc = new Arc(source, target);
		whitelist.remove(arc);
	}

	public Set<Arc> getWhitelist() {
		return whitelist;
	}

	public Arc getWhitelistArc(Variable source, Variable target) {
		for (Arc arc : whitelist) {
			if (arc.getSource().equals(source) && arc.getTarget().equals(target)) {
				return arc;
			}
		}
		return null;
	}

	/**
	 * add to blacklist of the project
	 */
	public void addBlacklist(Variable source, Variable target) {
		Arc arc = new Arc(source, target);
		blacklist.add(arc);
	}

	public void addBlacklist(Variable source, Variable target, String reason) {
		Arc arc = new Arc(source, target);
		arc.setReason(reason);
		blacklist.add(arc);
	}

	/**
	 * remove from blacklist of the project
	 */
	public void removeBlacklist(Variable source, Variable target) {
		Arc arc = new Arc(source, target);
		blacklist.remove(arc);
	}

	public Set<Arc> getBlacklist() {
		return blacklist;
	}

	public Arc getBlacklistArc(Variable source, Variable target) {
		for (Arc arc : blacklist) {
			if (arc.getSource().equals(source) && arc.getTarget().equals(target)) {
				return arc;
			}
		}
		return null;
	}


	public void addWhitelistDataType(String source, String target) {
		if (!dataTypesWhitelist.containsKey(source)) {
			dataTypesWhitelist.put(source, new ArrayList<>());
		}
		dataTypesWhitelist.get(source).add(target);
	}

	public void addBlacklistDataType(String source, String target) {
		if (!dataTypesBlacklist.containsKey(source)) {
			dataTypesBlacklist.put(source, new ArrayList<>());
		}
		dataTypesBlacklist.get(source).add(target);
	}

	public void removeWhitelistDataType(String source, String target) {
		if (dataTypesWhitelist.containsKey(source)) {
			dataTypesWhitelist.get(source).remove(target);
		}
	}

	public void removeBlacklistDataType(String source, String target) {
		if (dataTypesBlacklist.containsKey(source)) {
			dataTypesBlacklist.get(source).remove(target);
		}
	}

	public HashMap<String, List<String>> getDataTypesWhitelist() {
		return dataTypesWhitelist;
	}

	public HashMap<String, List<String>> getDataTypesBlacklist() {
		return dataTypesBlacklist;
	}
	public String getDesignID() {
		// TODO Auto-generated method stub
		return designID;
	}
	public void setProject(Project project) {
		this.project = project;
		
	}
	public String getDesignName() {
		return name;
	}
	public void setDesignName(String name) {
		this.name = name;
	}

	public String getDescription() {
		return designDescription;
	}

	public void setDescription(String description) {
		this.designDescription = description;
	}
	public Project getProject() {
		// TODO Auto-generated method stub
		return project;
	}

}
